@@ -1,4 +1,4 @@
-# Copyright (c) 2009-2018 - Simon Conseil
+# Copyright (c) 2009-2020 - Simon Conseil
 # Copyright (c) 2013      - Christophe-Marie Duquesne
 # Copyright (c) 2014      - Jonas Kaufmann
 # Copyright (c) 2015      - François D.
@@ -31,24 +31,23 @@ import os
 import pickle
 import random
 import sys
-import zipfile
-
-from click import progressbar, get_terminal_size
 from collections import defaultdict
 from datetime import datetime
 from itertools import cycle
 from os.path import isfile, join, splitext
 from urllib.parse import quote as url_quote
 
-from . import image, video, signals
-from .image import (process_image, get_exif_tags, get_exif_data, get_size,
-                    get_iptc_data)
+from click import get_terminal_size, progressbar
+from natsort import natsort_keygen, ns
+
+from . import image, signals, video
+from .image import get_exif_tags, get_image_metadata, get_size, process_image
 from .settings import get_thumb
-from .utils import (Devnull, copy, check_or_create_dir, url_from_path,
-                    read_markdown, cached_property, is_valid_html5_video,
-                    get_mime)
+from .utils import (Devnull, cached_property, check_or_create_dir, copy,
+                    get_mime, is_valid_html5_video, read_markdown,
+                    url_from_path)
 from .video import process_video
-from .writer import AlbumPageWriter, AlbumListPageWriter
+from .writer import AlbumListPageWriter, AlbumPageWriter
 
 
 class Media:
@@ -56,21 +55,24 @@ class Media:
 
     Attributes:
 
-    - ``type``: ``"image"`` or ``"video"``.
-    - ``filename``: Filename of the resized image.
-    - ``thumbnail``: Location of the corresponding thumbnail image.
-    - ``big``: If not None, location of the unmodified image.
-    - ``big_url``: If not None, url of the unmodified image.
-    - ``exif``: If not None contains a dict with the most common tags. For more
-        information, see :ref:`simple-exif-data`.
-    - ``raw_exif``: If not ``None``, it contains the raw EXIF tags.
+    :var Media.type: ``"image"`` or ``"video"``.
+    :var Media.filename: Filename of the resized image.
+    :var Media.thumbnail: Location of the corresponding thumbnail image.
+    :var Media.big: If not None, location of the unmodified image.
+    :var Media.big_url: If not None, url of the unmodified image.
 
     """
 
     type = ''
+    """Type of media, e.g. ``"image"`` or ``"video"``."""
 
     def __init__(self, filename, path, settings):
-        self.src_filename = self.filename = filename
+        self.filename = filename
+        """Filename of the resized image."""
+
+        self.src_filename = filename
+        """Filename of the resized image."""
+
         self.path = path
         self.settings = settings
         self.ext = os.path.splitext(filename)[1].lower()
@@ -82,14 +84,17 @@ class Media:
         self.thumb_path = join(settings['destination'], path, self.thumb_name)
 
         self.logger = logging.getLogger(__name__)
+
+        self.file_metadata = None
         self._get_metadata()
+
         # default: title is the filename
         if not self.title:
             self.title = self.filename
         signals.media_initialized.send(self)
 
     def __repr__(self):
-        return "<%s>(%r)" % (self.__class__.__name__, str(self))
+        return "<{}>({!r})".format(self.__class__.__name__, str(self))
 
     def __str__(self):
         return join(self.path, self.filename)
@@ -113,7 +118,8 @@ class Media:
             check_or_create_dir(orig_path)
             big_path = join(orig_path, self.src_filename)
             if not isfile(big_path):
-                copy(self.src_path, big_path, symlink=s['orig_link'])
+                copy(self.src_path, big_path, symlink=s['orig_link'],
+                     rellink=self.settings['rel_link'])
             return join(s['orig_dir'], self.src_filename)
 
     @property
@@ -148,10 +154,16 @@ class Media:
         return url_from_path(self.thumb_name)
 
     def _get_metadata(self):
-        """ Get image metadata from filename.md: title, description, meta."""
+        """Get image metadata from filename.md: title, description, meta."""
+
         self.description = ''
-        self.meta = {}
+        """Description extracted from the Markdown <imagename>.md file."""
+
         self.title = ''
+        """Title extracted from the Markdown <imagename>.md file."""
+
+        self.meta = {}
+        """Other metadata extracted from the Markdown <imagename>.md file."""
 
         descfile = splitext(self.src_path)[0] + '.md'
         if isfile(descfile):
@@ -171,52 +183,54 @@ class Image(Media):
 
     @cached_property
     def date(self):
+        """The date from the EXIF DateTimeOriginal metadata if available, or
+        from the file date."""
         return (self.exif and self.exif.get('dateobj', None) or
                 self._get_file_date())
 
     @cached_property
     def exif(self):
+        """If not `None` contains a dict with the most common tags. For more
+        information, see :ref:`simple-exif-data`.
+        """
         datetime_format = self.settings['datetime_format']
         return (get_exif_tags(self.raw_exif, datetime_format=datetime_format)
                 if self.raw_exif and self.ext in ('.jpg', '.jpeg') else None)
 
     def _get_metadata(self):
-        super(Image, self)._get_metadata()
+        super()._get_metadata()
+        self.file_metadata = get_image_metadata(self.src_path)
+
         # If a title or description hasn't been obtained by other means, look
         #  for the information in IPTC fields
         if self.title and self.description:
             # Nothing to do - we already have title and description
             return
 
-        try:
-            iptc_data = get_iptc_data(self.src_path)
-        except Exception as e:
-            self.logger.warning('Could not read IPTC data from %s: %s',
-                                self.src_path, e)
-        else:
-            if not self.title and iptc_data.get('title'):
-                self.title = iptc_data['title']
-            if not self.description and iptc_data.get('description'):
-                self.description = iptc_data['description']
+        iptc_data = self.file_metadata['iptc']
+        if not self.title and iptc_data.get('title'):
+            self.title = iptc_data['title']
+        if not self.description and iptc_data.get('description'):
+            self.description = iptc_data['description']
 
     @cached_property
     def raw_exif(self):
-        try:
-            return (get_exif_data(self.src_path)
-                    if self.ext in ('.jpg', '.jpeg') else None)
-        except Exception as e:
-            self.logger.warning('Could not read EXIF data from %s: %s',
-                                self.src_path, e)
+        """If not `None`, contains the raw EXIF tags."""
+        if self.ext in ('.jpg', '.jpeg'):
+            return self.file_metadata['exif']
 
     @cached_property
     def size(self):
+        """The dimensions of the resized image."""
         return get_size(self.dst_path)
 
     @cached_property
     def thumb_size(self):
+        """The dimensions of the thumbnail image."""
         return get_size(self.thumb_path)
 
     def has_location(self):
+        """True if location information is available for EXIF GPSInfo."""
         return self.exif is not None and 'gps' in self.exif
 
 
@@ -226,7 +240,7 @@ class Video(Media):
     type = 'video'
 
     def __init__(self, filename, path, settings):
-        super(Video, self).__init__(filename, path, settings)
+        super().__init__(filename, path, settings)
         base, ext = splitext(filename)
         self.src_filename = filename
         self.date = self._get_file_date()
@@ -247,10 +261,10 @@ class Album:
 
     :var description_file: Name of the Markdown file which gives information
         on an album
-    :ivar index_url: URL to the index page.
-    :ivar output_file: Name of the output HTML file
-    :ivar meta: Meta data from the Markdown file.
-    :ivar description: description from the Markdown file.
+    :var index_url: URL to the index page.
+    :var output_file: Name of the output HTML file
+    :var meta: Meta data from the Markdown file.
+    :var description: description from the Markdown file.
 
     For details how to annotate your albums with meta data, see
     :doc:`album_information`.
@@ -304,12 +318,12 @@ class Album:
         signals.album_initialized.send(self)
 
     def __repr__(self):
-        return "<%s>(path=%r, title=%r)" % (self.__class__.__name__, self.path,
-                                            self.title)
+        return "<{}>(path={!r}, title={!r})".format(
+            self.__class__.__name__, self.path, self.title)
 
     def __str__(self):
-        return ('{} : '.format(self.path) +
-                ', '.join('{} {}s'.format(count, _type)
+        return (f'{self.path} : ' +
+                ', '.join(f'{count} {_type}s'
                           for _type, count in self.medias_count.items()))
 
     def __len__(self):
@@ -359,14 +373,15 @@ class Album:
                 root_path = self.path if self.path != '.' else ''
                 if albums_sort_attr.startswith("meta."):
                     meta_key = albums_sort_attr.split(".", 1)[1]
-                    key = lambda s: locale.strxfrm(
-                        self.gallery.albums[join(root_path, s)].meta.get(meta_key, [''])[0])
+                    key = natsort_keygen(key=lambda s:
+                        self.gallery.albums[join(root_path, s)].meta.get(meta_key, [''])[0],
+                        alg=ns.LOCALE)
                 else:
-                    key = lambda s: locale.strxfrm(
-                        getattr(self.gallery.albums[join(root_path, s)],
-                                albums_sort_attr))
+                    key = natsort_keygen(key=lambda s:
+                        getattr(self.gallery.albums[join(root_path, s)], albums_sort_attr),
+                        alg=ns.LOCALE)
             else:
-                key = locale.strxfrm
+                key = natsort_keygen(alg=ns.LOCALE)
 
             self.subdirs.sort(key=key,
                               reverse=self.settings['albums_sort_reverse'])
@@ -379,9 +394,11 @@ class Album:
                 key = lambda s: s.date or datetime.now()
             elif medias_sort_attr.startswith('meta.'):
                 meta_key = medias_sort_attr.split(".", 1)[1]
-                key = lambda s: locale.strxfrm(s.meta.get(meta_key, [''])[0])
+                key = natsort_keygen(key=lambda s: s.meta.get(meta_key, [''])[0],
+                                    alg=ns.LOCALE)
             else:
-                key = lambda s: locale.strxfrm(getattr(s, medias_sort_attr))
+                key = natsort_keygen(key=lambda s: getattr(s, medias_sort_attr),
+                                    alg=ns.LOCALE)
 
             self.medias.sort(key=key,
                              reverse=self.settings['medias_sort_reverse'])
@@ -442,7 +459,7 @@ class Album:
                     # fallback to the size of src_path if dst_path is missing
                     size = f.size
                     if size is None:
-                        size = get_size(f.src_path)
+                        size = f.file_metadata['size']
 
                     if size['width'] > size['height']:
                         self._thumbnail = (url_quote(self.name) + '/' +
@@ -520,42 +537,15 @@ class Album:
 
     @cached_property
     def zip(self):
-        """Make a ZIP archive with all media files and return its path.
-
-        If the ``zip_gallery`` setting is set,it contains the location of a zip
-        archive with all original images of the corresponding directory.
-
+        """Placeholder ZIP method.
+        The ZIP logic is controlled by the zip_gallery plugin
         """
-        zip_gallery = self.settings['zip_gallery']
-
-        if zip_gallery and len(self) > 0:
-            zip_gallery = zip_gallery.format(album=self)
-            archive_path = join(self.dst_path, zip_gallery)
-            if (self.settings.get('zip_skip_if_exists', False) and
-                    isfile(archive_path)):
-                self.logger.debug("Archive %s already created, passing",
-                                  archive_path)
-                return zip_gallery
-
-            archive = zipfile.ZipFile(archive_path, 'w', allowZip64=True)
-            attr = ('src_path' if self.settings['zip_media_format'] == 'orig'
-                    else 'dst_path')
-
-            for p in self:
-                path = getattr(p, attr)
-                try:
-                    archive.write(path, os.path.split(path)[1])
-                except OSError as e:
-                    self.logger.warn('Failed to add %s to the ZIP: %s', p, e)
-
-            archive.close()
-            self.logger.debug('Created ZIP archive %s', archive_path)
-            return zip_gallery
+        return None
 
 
-class Gallery(object):
+class Gallery:
 
-    def __init__(self, settings, ncpu=None):
+    def __init__(self, settings, ncpu=None, quiet=False):
         self.settings = settings
         self.logger = logging.getLogger(__name__)
         self.stats = defaultdict(int)
@@ -570,7 +560,8 @@ class Gallery(object):
         ignore_files = settings['ignore_files']
 
         progressChars = cycle(["/", "-", "\\", "|"])
-        show_progress = (self.logger.getEffectiveLevel() >= logging.WARNING and
+        show_progress = (not quiet and
+                         self.logger.getEffectiveLevel() >= logging.WARNING and
                          os.isatty(sys.stdout.fileno()))
         self.progressbar_target = None if show_progress else Devnull()
 
@@ -612,7 +603,8 @@ class Gallery(object):
                 album.create_output_directories()
                 albums[relpath] = album
 
-        print("\rCollecting albums, done.")
+        if show_progress:
+            print("\rCollecting albums, done.")
 
         with progressbar(albums.values(), label="%16s" % "Sorting albums",
                          file=self.progressbar_target) as progress_albums:
