@@ -48,6 +48,7 @@ them for good.
 
 import logging
 import os
+
 from sigal import signals
 
 logger = logging.getLogger(__name__)
@@ -104,7 +105,7 @@ def filter_nomedia(album, settings=None):
             album.medias = []
 
         else:
-            with open(nomediapath, "r") as nomediaFile:
+            with open(nomediapath) as nomediaFile:
                 logger.info("Found a .nomedia file in %s, ignoring its "
                             "entries", album.name)
                 ignored = nomediaFile.read().split("\n")
