@@ -1,5 +1,5 @@
 # Copyright (c)      2013 - Christophe-Marie Duquesne
-# Copyright (c) 2013-2018 - Simon Conseil
+# Copyright (c) 2013-2020 - Simon Conseil
 
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to
@@ -27,7 +27,7 @@ import subprocess
 from os.path import splitext
 
 from . import image, utils
-from .settings import get_thumb, Status
+from .settings import Status, get_thumb
 from .utils import is_valid_html5_video
 
 
