@@ -1,4 +1,4 @@
-# Copyright (c) 2009-2018 - Simon Conseil
+# Copyright (c) 2009-2020 - Simon Conseil
 # Copyright (c)      2013 - Christophe-Marie Duquesne
 # Copyright (c)      2018 - Edwin Steele
 
@@ -20,25 +20,25 @@
 # FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 # IN THE SOFTWARE.
 
-import jinja2
-import logging
 import imp
+import logging
 import os
 import sys
 import types
-
 from distutils.dir_util import copy_tree
-from jinja2 import Environment, FileSystemLoader, ChoiceLoader, PrefixLoader
+
+import jinja2
+from jinja2 import ChoiceLoader, Environment, FileSystemLoader, PrefixLoader
 from jinja2.exceptions import TemplateNotFound
 
-from .pkgmeta import __url__ as sigal_link
+from . import signals
 from .utils import url_from_path
 
 THEMES_PATH = os.path.normpath(os.path.join(
     os.path.abspath(os.path.dirname(__file__)), 'themes'))
 
 
-class AbstractWriter(object):
+class AbstractWriter:
     template_file = None
 
     def __init__(self, settings, index_title=''):
@@ -88,8 +88,8 @@ class AbstractWriter(object):
         try:
             self.template = env.get_template(self.template_file)
         except TemplateNotFound:
-            self.logger.error('The template %s was not found.',
-                              self.template_file)
+            self.logger.error('The template %s was not found in template folder %s.',
+                              self.template_file, theme_relpath)
             sys.exit(1)
 
         # Copy the theme files in the output dir
@@ -99,6 +99,7 @@ class AbstractWriter(object):
     def generate_context(self, album):
         """Generate the context dict for the given path."""
 
+        from . import __url__ as sigal_link
         self.logger.info("Output album : %r", album)
         return {
             'album': album,
@@ -112,8 +113,9 @@ class AbstractWriter(object):
 
     def write(self, album):
         """Generate the HTML page and save it."""
-
-        page = self.template.render(**self.generate_context(album))
+        context = self.generate_context(album)
+        signals.before_render.send(context)
+        page = self.template.render(**context)
         output_file = os.path.join(album.dst_path, album.output_file)
 
         with open(output_file, 'w', encoding='utf-8') as f:
